      subroutine db_get_co2cld(cfgname, line, pixel, obs_clr, tcold, 
     &           twarm, pct, eca, ncloudy, ngood_co2, month, qual_flag, 
     &           conf_flag, hc_flag, ci_flag )

c!F77------------------------------------------------------------------
c!Description: Compute cloud top pressure and effective cloud amount 
c              from MODIS IR data.
c
c!Input parameters:
c cfgname                    Configuration file name
c line                       Line number within a swath (1-10)
c pixel                      Pixel number within a 1km scan (1-1500)
c obs_clr                    Observed clear sky pixels
c                              0=none present
c                              1=present
c                              use only with mod06ct_proc_opt = 2
c tcold                      Obs cloud or cold brightness temperatures
c                              (K) for MODIS IR bands 36, 35, 34, 33, 31
c twarm                      Obs clear or warm brightness temperatures
c                              (K) for MODIS IR bands 36, 35, 34, 33, 31
c ncloudy                    Number of clouds 
c ngood_co2                  Number of valid pixels
c month                      Number data month (1-12)
c
c The following in COMMON /MOD06_DATA/ are used:
c wprof1                     Water vapor mixing ratio profile (g/kg)
c tprof1                     Temperature profile (K)
c tem1                       Surface temperature (K)
c view                       Satellite zenith angle (deg)
c cs_bias_corr               clear-sky radiance bias correction 
c bad_value                  missing or bad data flag
c
c!Output parameters:
c pct                        Cloud top pressure (hPa)
c eca                        Cloud effective emissivity
c qual_flag                  Product quality flag 
c                             (0=not useful, 1=useful)
c conf_flag                  Product confidence flag 
c                             (0-3, 0=worst, 3=best)
c hc_flag                    Indicates whether or not high cloud was
c                              detected (0=missing, 1=no, 2=yes)
c ci_flag                    Indicates whether or not cirrus cloud was
c                              detected (0=missing, 1=no, 2=yes)
c ncloudy                    Number cloudy pixels in box
c ngood_co2                  Number of valid pixels for co2 retrievals
c
c The following are output to COMMON /MOD06_DATA/:
c height_tropopause     Pressure of tropopause from temperature profile
c cloud_h_method        Number of the selected ratio solution
c cloudtop_pre_ir       Window cloud top height (mb)
c spec_cloud_forcing    Cloud-clear or cold-warm radiance for MODIS 
c                       IR bands 36, 35, 34, 33, 31 (mW/m2/sr/cm-1)
c cloudtop_pres         Final cloud top height pressure (mb)
c cloudtop_temp         Cloud top temperature from temperature profile
c cloudtop_eff_emi           Effective cloud amount (no units)
c cloudtop_pres_from_ratios  Cloud top pressure for each of 5 ratios
c
c!Revision history:
c  R. Frey, H. Zhang 12-20-04
c  Use average cloud radiance from cloud pixels, cloud fraction used 
c  as ECA, also effect ECA from CO2 algorithm, no ratio 33/31 used,
c  also no ratio 35/33 for aqua. Took off the NEDR check at
c  11um for starting the co2 slicing algorithm, no low clouds quality 
c  control. Selected bands used in finding the final co2 heights, 
c  band 33, 35 and 36 for c  Terra, band 33 through 36 for Aqua.
c
c R. Frey  06-11-04
c Set Aqua "rmins" to 1.0, implemented 7.4 forward model, use more
c atmospheric levels in GDAS profiles as input, added surface 
c emissivities, reduced number of forward model calculations
c 05-05-04 R. Frey, P. Hubanks, R. Hucek
c Implemented fix to Cirrus Flag
c R. Frey  10-17-03
c Implemented 101-level processing, climatological ozone profiles,
c atmospherically corrected window channel cloud pressures, check on
c cloud pressure functions, modified Aqua "rmins" for Aqua Collection 4
c delivery.  
c
c calls:
c function modrad  Compute MODIS IR TOA radiance given transmittance
c function modis_planck Compute Planck radiance for MODIS IR bands
c function modis_bright Compute brightness temperature for MODIS IR 
c bands
c subroutine fm_tran_modisd101   Compute transmittance profile for MODIS 
c IR bands
c subroutine clozo101   Computes ozone concentration profile from 
c  climatology
c assign_eco_emis.f
c getiremis.f
c fm_modrad_emis.f
c
c!Team-unique Header:
c
c!End------------------------------------------------------------------

      implicit none
      save

      include 'db_mod06uw_data.inc'
      include 'db_mod06uw_debug.inc'
      include 'platform_name.inc'

c ... scalar arguments
      real pct,eca
      integer qual_flag,conf_flag,ci_flag,hc_flag,line,pixel,obs_clr,
     &        ncloudy, ngood_co2, month

c ... array arguments
      character*(*) cfgname
      real tcold(ntbct),twarm(ntbct)

c ... local scalars
      real bot,top,db,fm,fm1,fm2,fmsav,ratio,rtemin,sum,psfc,
     &     tmin,tw,pfco2,ecawin,ecaco2,ptrp,pwin,tct,ts,theta,tss,
     &     rlat,emis_out,rho_out,emiswc,ppsfc,
     &     emisrw,rtmp1,rtmp2
      integer id,is1,iw1,k,k1,k2,ll,ltrp,lwin,ip,mid_px,
     &        isp,lco2,ipco2,ngch,ipw,iout,jout,mid_ln,ii,jj,
     &        iisp,kban,jdet,imet,jmet,last_imet,last_jmet,
     &        lsf,landsea,init,iview,last_iview,d_theta
      logical ok, neg, start, new_prof
   
      integer nb_wavelen
      parameter (nb_wavelen = 7)
      real*4 emis(nb_wavelen),rho(nb_wavelen),freqemis(nb_wavelen)

c ... local arrays
      real amo(nsct),ra(plev,nbct),rclr(nbct),robs(nbct),rterms(nsct),
     &     ozone(plev),pp(plev),tp(plev),ttpp(plev),
     &     taup(plev,nbct),rmin(nbct),rwarm(nbct),rwcld(nsct),
     &     em_adj(nsct),delr(nbct),w(plev),pco2(nsct),
     &     bias(nbct),rmin_Terra(nbct),rmin_Aqua(nbct),rclr_s(plev),
     &     freq_terra(nbct),freq_aqua(nbct),sfc_emis(nbct),freq(nbct),
     &     rc1(nbct),rc2(nbct)
      integer lev(nsct),mbnds(nbct),
     &     krto(nsct),mcsbnds(nbct),kch(nsct,2)
      integer ratio_sum, ratio_terra(4), ratio_aqua(3), ratio_sat(4) 

c ... external functions
      real modis_bright, modis_planck, fm_modrad_emis
      external modis_bright, modis_planck, fm_modrad_emis

c ... external subroutines
      external fm_tran_modisd101, clozo101, getiremis,
     &         assign_eco_emis
c ... -----------------------------------------------------------------

c ... data statements

c ... Define 101 pressure levels.
      data pp      / 0.0050,    0.0161,    0.0384,    0.0769,    0.1370,
     +    0.2244,    0.3454,    0.5064,    0.7140,    0.9753,    1.2972,
     +    1.6872,    2.1526,    2.7009,    3.3398,    4.0770,    4.9204,
     +    5.8776,    6.9567,    8.1655,    9.5119,   11.0038,   12.6492,
     +   14.4559,   16.4318,   18.5847,   20.9224,   23.4526,   26.1829,
     +   29.1210,   32.2744,   35.6505,   39.2566,   43.1001,   47.1882,
     +   51.5278,   56.1260,   60.9895,   66.1253,   71.5398,   77.2396,
     +   83.2310,   89.5204,   96.1138,  103.0172,  110.2366,  117.7775,
     +  125.6456,  133.8462,  142.3848,  151.2664,  160.4959,  170.0784,
     +  180.0183,  190.3203,  200.9887,  212.0277,  223.4415,  235.2338,
     +  247.4085,  259.9691,  272.9191,  286.2617,  300.0000,  314.1369,
     +  328.6753,  343.6176,  358.9665,  374.7241,  390.8926,  407.4738,
     +  424.4698,  441.8819,  459.7118,  477.9607,  496.6298,  515.7200,
     +  535.2322,  555.1669,  575.5248,  596.3062,  617.5112,  639.1398,
     +  661.1920,  683.6673,  706.5654,  729.8857,  753.6275,  777.7897,
     +  802.3714,  827.3713,  852.7880,  878.6201,  904.8659,  931.5236,
     +  958.5911,  986.0666, 1013.9476, 1042.2319, 1070.9170, 1100.0000/

c ... Define indices of channel combinations used in the co2-slicing
c ... method.
      data kch /1,2,2,3,2,3,4,4/

c ... Define ratios used in the co2 algorithm
      data ratio_terra/1,2,3,4/
      data ratio_aqua/1,2,4/

c ... Initialize ozone profile.
      data ozone/plev*0.0/

c ... Order of MODIS IR band numbers used
      data mbnds/36,35,34,33,31/      

c ... Define wavenumbers of CO2 bands used (36, 35, 34, 33, 31).
      data freq_terra /7.045367E+02, 7.188681E+02, 7.308963E+02,
     &                 7.483394E+02, 9.080884E+02/
      data freq_aqua  /7.035020E+02, 7.182089E+02, 7.307761E+02,
     &                 7.482977E+02, 9.076808E+02/

c ... Order of clear-sky radiance bias values used.
      data mcsbnds/5,4,3,2,1/

c ... Ice cloud emissivity adjustments for each channel combination.
      data em_adj / 1.0, 1.0, 1.0, 1.0 /

c ... Define noise level (in radiance units -- mw/m2/ster/cm-1) for each CO2 channel.
      data rmin_Aqua /-1.0, -1.0, -1.0, -1.0, -0.5/  
      data rmin_Terra /-2.5, -5.0, -20.0, -5.0, -0.5/  

c     Dectector number for transmittance calculations (0=averaged).
      data jdet /0/

c     Initialize input GDAS profile indecis.
      data imet /-1/, jmet /-1/, last_iview /-1/

c     Initialize 'init' flag
      data init /0/

c     Black cloud emissivity
      data emisrw /1.0/

c ... -----------------------------------------------------------------

c ... Perform one time initializations.

      if(init .eq. 0) then

c ...   Check the platform name (it should be 'Terra' or 'Aqua')
        if (platform_name(1:5) .eq. 'Terra') then
          ratio_sum = 4 
          do k = 1, nbct
            rmin(k) = rmin_Terra(k)
            freq(k) = freq_terra(k)
          enddo 
          do k = 1, ratio_sum
            ratio_sat(k) = ratio_terra(k)
          enddo 
        else if (platform_name(1:4) .eq. 'Aqua') then
          ratio_sum = 3
          do k = 1, nbct
            rmin(k) = rmin_Aqua(k)
            freq(k) = freq_aqua(k)
          enddo 
          do k = 1, ratio_sum
            ratio_sat(k) = ratio_aqua(k)
          enddo 
        end if 

        init = 1

      end if

c ... -----------------------------------------------------------------

c ... Initializations

      new_prof = .false.
      do id = 1,nsct
        krto(id) = 0
        lev(id) = 0
      enddo

c ... -----------------------------------------------------------------

c ... Define output array indices for the current line and pixel.
      iout = (pixel / isamp) + 1
      jout = (line / isamp) + 1

c ... Define mid-point of box in terms of line and pixel.
      mid_px = pixel + isamp / 2
      mid_ln = line + isamp / 2

c ... Get satellite zenith angle.
      theta = view(mid_px,mid_ln)
      if(iout .gt. 675 .and. iout .lt. 679 .and. theta .lt. 0.0) then
        theta = 0.1
      end if
      iview = int(theta + 0.5)
      d_theta = abs(iview - last_iview)

c ... Get current latitude.
      rlat = lat1(mid_px,mid_ln)

c ... Recover i,j position of met data.
      imet = met_data_i(mid_px,mid_ln)
      jmet = met_data_j(mid_px,mid_ln)

c ... -----------------------------------------------------------------

c ... Check if using a previously accessed temperature and moisture 
c     profile.  If not, perform necessary calculations.

      if( (imet .ne. last_imet .or. jmet .ne. last_jmet) .or.
     &    (d_theta .gt. 0) ) then

        new_prof = .true.
        last_iview = iview
        last_imet = imet
        last_jmet = jmet

c ... -----------------------------------------------------------------

c       Get climatological ozone profile.
        call clozo101(rlat, month, ozone)

c ...   Get temperature and water vapor mixing ratio profiles.
        do ii = 1,plev
          tp(ii) = tprof1(ii,mid_px,mid_ln)
          w(ii) = wprof1(ii,mid_px,mid_ln)
        end do

c ...   Calculate transmittance profiles (101 level fast model)
        do k = 1, nbct
          kban = mbnds(k)
          call fm_tran_modisd101(cfgname, tp, w, ozone, theta, kban, 
     &         jdet, taup(1,k))
        enddo

c ...   Write debug information.
        if(debug .gt. 0) then
          if(line .eq. 6 .and. pixel .eq. 676) then
            write(h_output,'(''Atm level, pressure, temperature, '',
     &        ''ozone, transmittances'')')
            do k=1,plev
              write(h_output,'(i4,8f10.4)') k,pp(k),tp(k),ozone(k),
     &            (taup(k,jj),jj=1,nbct)
            enddo
          endif
        end if

c ... -----------------------------------------------------------------
 
c ...   Find surface level.
c ...   Get surface pressure.
        psfc = pre1(mid_px,mid_ln)
        do ll = 1,plev
          if(psfc .le. pp(ll)) then
            isp = ll
            go to 100
          end if
        enddo
 100    continue

c ...   Get surface temperature.
        ts = tem1(mid_px,mid_ln)

c ...   Get land/sea tag.
        lsf = land1(mid_px,mid_ln)
        if(lsf .eq. 1 .or. lsf .eq. 2 .or. lsf .eq. 4) then
          landsea = 1
        else
          landsea = 0
        end if

c ...   Get surface emissivity.

        call assign_eco_emis(landsea,emis,rho,freqemis)
c       write(*,'(7f12.3)') emis,rho,freqemis
        do k = 1, nbct
          call getiremis(nb_wavelen,freqemis,freq(k),emis,rho,emis_out,
     &                   rho_out)
          sfc_emis(k) = emis_out
        enddo
c       write(*,'(5f12.3)') sfc_emis

c ...   Find level of coldest temperature between 100 mb and the surface.
        tmin = 350.0
        do ll = 44,isp
          if(tp(ll) .le. tmin) then
            tmin = tp(ll)
            ltrp = ll
          end if
        enddo
        ptrp = pp(ltrp)

c ...   May 2003  Hong Zhang
c ...   Get IRW Radiance profile instead of Temperature profile
        Do  k = ltrp,isp
          if(k .eq. isp) then
            emiswc = sfc_emis(kwc)
            ppsfc = psfc
          else
            emiswc = 1.0
            ppsfc = pp(k)
          end if
          tss = tp(k)
          iisp = k
          rclr_s(k) = fm_modrad_emis(tss,emiswc,ppsfc,pp,tp,taup(1,kwc),
     &                             mbnds(kwc),iisp)
          TTPP(k) = modis_bright(rclr_s(k),mbnds(kwc),0)
        enddo
  
      end if

c ... -----------------------------------------------------------------

c ... Compute the "window" cloud height.
      tw = tcold(kwc)
      do ll = ltrp,isp
        if(ttpp(ll) .lt. tw) then
          lwin = ll
        end if
      enddo


c ... Find which level tw is closest to and define beginning level at
c ... which to begin comparison of LHS and RHS of co2-slicing equation.
      if(lwin .eq. isp) then
        iw1 = lwin - 2
      else if( abs(tw - ttpp(lwin)) .gt. abs(tw - ttpp(lwin + 1)) ) then
        lwin = lwin + 1
        iw1 = lwin - 2
      else
        iw1 = lwin - 1
      end if

c ... Define all output values for window cloud height retrieval.
      pwin = pp(lwin)
      ecawin = float(ncloudy) / ngood_co2 
      ipw = 6
c ... Round cloud pressure to nearest 5 mb.
      pwin = (nint(pwin / 5.0)) * 5.0
      pct = pwin
      eca = ecawin
      ip = ipw
      tct = tp(lwin)

c ... High Cloud Flag Initialization using Window Retrieval values.
c ... hc_flag values may be overwritten with CO2 retrieval values below.
      if(pwin .lt. 400.0) then
        hc_flag = 2
      else
        hc_flag = 1
      end if

c ... Bug Fix Implemented on 5/5/2004
c ... Cirrus Flag Initialization using Window Retrieval values.
c ... ci_flag values may be overwritten with CO2 retrieval values below.
      if(pwin .le. 700.0 .and. eca .le. 0.95) then
        ci_flag = 2
      else
        ci_flag = 1
      end if

c ... For L3 purposes, label all retrievals the same quality (3).
      qual_flag = 1
      conf_flag = 3

c ... -----------------------------------------------------------------

c ... Fill all output arrays which pertain to the "window" cloud height
c ... retrieval. Some values may be overwritten below.
      cloud_h_method(iout,jout) = ip * 1.0
      cloudtop_pres(iout,jout) = pct
      cloudtop_temp(iout,jout) = tct
      height_tropopause(iout,jout) = (nint(ptrp / 5.0)) * 5.0
      cloudtop_eff_emi(iout,jout) = eca
      cloudtop_pre_ir(iout,jout) = pwin

c ... -----------------------------------------------------------------

c ... Write debug information about input data and IR "window" cloud
c ... height retrieval.
      if(debug .gt. 1) then
       if(line .eq. 6 .and. pixel .eq. 676) then
        write(h_output,'(1x)')
        write(h_output,'(''get_co2cld debug'')')
        write(h_output,'(''Line and pixel '',2i5)') line,pixel
        write(h_output,'(''Output array indices are '',2i10)')
     &      iout,jout
        write(h_output,'(''Mid-point of box is at '',2i10)')
     &      mid_ln,mid_px
        write(h_output,'(''met and vza indecis '',3i10)')
     &      imet, jmet, iview
        write(h_output,'(1x)')
        write(h_output,'(''Input data / window height retrieval'')')
        write(h_output,'(''Trop level and pressure '',i5,f10.3)')
     &      ltrp,ptrp
        write(h_output,'(''Surface level and pressure '',i5,f10.3)')
     &      isp,psfc
        write(h_output,'(''Surface level calc Tb (31) '',f10.3)')
     &      ttpp(isp)
        write(h_output,'(''Window cloud height level '',i4)') lwin
        write(h_output,'(''Window cloud fraction '',f10.3)') ecawin 
        write(h_output,'(''Window cloud height pressure '',
     &      ''and temperature '',2f10.3)') pwin,tct
       end if
      end if

c ... -----------------------------------------------------------------
  
c ... If using observed clear-sky radiances, make sure a valid clear-
c ... sky radiance was found.

      if( (mod06ct_proc_opt .ne. 2 ) .or.
     &    (mod06ct_proc_opt .eq. 2 .and. obs_clr .eq. 1) ) then

c ... -----------------------------------------------------------------
  
c ... Check if using new profile data.
      if(new_prof) then

c ...   Perform radiative transfer calculations for co2-slicing method.

c ...   Define index of first level of integration in computing RHS of
c ...   co2-slicing equation.

        is1 = isp - 1

        do k = 1,nbct

c ...     Compute TOA clear radiance from input profiles.
          rclr(k) = fm_modrad_emis(ts,sfc_emis(k),psfc,pp,tp,taup(1,k),
     &                             mbnds(k),isp)

c ...     Compute components of RHS 
          sum = 0.0
          do ll = is1,1,-1
            db = modis_planck(tp(ll + 1),mbnds(k),0)
     &               - modis_planck(tp(ll),mbnds(k),0)
            sum = sum + 0.5 * (taup(ll + 1,k) + taup(ll,k)) * db
            ra(ll,k) = sum
          enddo

        enddo

      end if

c ... -----------------------------------------------------------------
  
c ... Apply clear-sky radiance bias corrections and calculate spectral
c ... cloud forcing.

      do k = 1,nbct
      
c ...   Get bias correction for this channel
        bias(k) = cs_bias_corr(mid_px,mid_ln,mcsbnds(k))

c ...   Get "warm" radiance value from either calculation or observations.
        if(mod06ct_proc_opt .eq. 1) then
c ...     Apply clear-sky radiance bias correction.
          if(bias(k) .ne. bad_value) then
            rtmp1 = modis_bright(rclr(k),mbnds(k),0)
            rc1(k) = modis_planck(rtmp1,mbnds(k),1)
c           rc2(k) = rc1(k) + bias(k)
            rc2(k) = rc1(k) + 0.0
            rtmp2 = modis_bright(rc2(k),mbnds(k),1)
            rwarm(k) = modis_planck(rtmp2,mbnds(k),0)
          else
c ...       No correction is available.
            rwarm(k) = rclr(k)
          end if
        else
          rwarm(k) = modis_planck(twarm(k),mbnds(k),0)
        end if

c ...   Get "cold" or "cloudy" radiance from observations.
        robs(k) = modis_planck(tcold(k),mbnds(k),0)

c ...   Calculate "cold - warm" or "clear - cloudy" value. These are 
c ...   components of the LHS of the co2-slicing equation.
        delr(k) = robs(k) - rwarm(k)
        spec_cloud_forcing(iout,jout,k) = delr(k)

      enddo

c ... -----------------------------------------------------------------
    
c ... Write debug information for radiative transfer calculations.
      if(debug .gt. 0) then
       if(line .eq. 6 .and. pixel .eq. 676) then
        write(h_output,'(1x)')
        write(h_output,'(''Radiative transfer section'')')
        write(h_output,'(''Surface temperature '',f10.3)') ts
        write(h_output,'(''Values for MODIS bands '',5i10)')
     &       (mbnds(ii),ii=1,nbct)
        write(h_output,'(''Calculated TOA radiances '',5f10.3)')
     &       (rclr(ii),ii=1,nbct)
        write(h_output,'(''Clear-sky radiance bias  '',5f10.3)')
     &       (bias(ii),ii=1,nbct)
        write(h_output,'(''Warm or clear radiances  '',5f10.3)')
     &       (rwarm(ii),ii=1,nbct)
        write(h_output,'(''Cold or cloudy radiances '',5f10.3)')
     &       (robs(ii),ii=1,nbct)
        write(h_output,'(''Spectral cloud forcing  '',5f10.3)')
     &       (delr(ii),ii=1,nbct)
       end if
      end if

c ... -----------------------------------------------------------------
  
c ...   Compute cloud height for each channel combination (when possible).
 
        do id = 1,nsct

          krto(id) = 0
          fmsav = 1000.0
          k1 = kch(id,1)
          k2 = kch(id,2)

c ...     Check if values of cold minus warm are within instrument noise.
          if(delr(k1) .le. rmin(k1)) then
            if(delr(k2) .le. rmin(k2)) then
      
c ...           Find minimum difference between LHS and RHS of co2-slicing
c ...           equation. Apply emissivity correction.

                ok = .false. 
                do ll = iw1,ltrp,-1

                  fm1 = delr(k1) / delr(k2)
                  fm2 = (ra(ll,k1) * em_adj(id)) / ra(ll,k2)
                  fm = fm1 - fm2
                  if(fm .lt. 0.0) then
                    neg = .true.
                  else
                    neg = .false.
                  end if
                  if(ll .eq. iw1) start = neg
                  if(neg .neqv. start) then
 
                    ok = .true.
                    lev(id) = ll
                    fmsav = abs(fm)
                    go to 2000
                  end if

                enddo
 2000           continue

                if(fmsav .lt. 1000.0) then
                  krto(id) = 1
                end if
     
                if( (.not. ok) ) krto(id) = 0
                if( (.not. ok) .and. neg) then
                  krto(id) = 1
                  lev(id) = iw1 
                end if

            end if
          end if

        enddo

        if(debug .gt. 0) then
         if(line .eq. 6 .and. pixel .eq. 676) then
          write(h_output,'(1x)')
          write(h_output,'(''Initial CO2-slicing cloud heights'')')
          write(h_output,'(''MODIS channel combinations:'')')
          write(h_output,'(8x,8(i8,''/'',i2))') (mbnds(kch(ii,1)),
     &       mbnds(kch(ii,2)),ii=1,nsct)
          write(h_output,'(''atm lev '',8i10)') (lev(ii),ii=1,nsct)
         end if
        end if

c ...   ---------------------------------------------------------------
  
c ...   Compute effective cloud amounts for every channel combination
c ...   for which there was a successful cloud height retrieval.
        do id = 1,nsct
          rwcld(id) = bad_value
          amo(id) = bad_value
          if(krto(id) .ne. 0) then
            ll = lev(id)
            rwcld(id) = fm_modrad_emis(tp(ll),emisrw,pp(ll),pp,tp,
     &                  taup(1,kwc),mbnds(kwc),ll)
            bot = rwcld(id) - rwarm(kwc)
            top = delr(kwc)
            if(abs(bot) .gt. 0.1) then
              ratio = top / bot
              if(ratio .le. 1.0 .and. ratio .ge. 0.01) then
                amo(id) = ratio * ecawin
              else
                krto(id) = 0
              end if
            else
              krto(id) = 0
            end if
          end if
        enddo
     
c ...   ---------------------------------------------------------------
	  
c ...   Write debug information about effective emissivity calculations
        if(debug .gt. 0) then
         if(line .eq. 6 .and. pixel .eq. 676) then
          write(h_output,'(1x)')
          write(h_output,'(''Initial CO2-slicing eff. emiss.'')')
          write(h_output,'(''MODIS channel combinations:'')')
          write(h_output,'(7x,8(i8,''/'',i2))') (mbnds(kch(ii,1)),
     &       mbnds(kch(ii,2)),ii=1,nsct)
          write(h_output,'(''cld rad   '',8f10.3)')
     &       (rwcld(ii),ii=1,nsct)
          write(h_output,'(''clr rad, delr '',2f10.3)') rwarm(kwc),
     &       delr(kwc)
          write(h_output,'(''eff emiss '',8f10.3)')
     &       (amo(ii),ii=1,nsct)
         end if
        end if
  
c ...   ---------------------------------------------------------------
  
c ...   Find the cloud height that best satisfies the radiative transfer.
c ...   12/8/2004, no band 31 for both Terra and Aqua
c ...              Terra: band 33, 35 and 36 
c ...              Aqua: no ratio 35/33, use band 33 through 36

        if (platform_name(1:5) .eq. 'Terra') then
          do id = 1,nsct 
          if(krto(id) .ne. 0 ) then
            rterms(id) = 0.0
            ll = lev(id)
            do k = 1, nbct-1
              if (k .ne. 3 ) then
              rterms(id) = rterms(id) + (robs(k) - rwarm(k) -
     &                          amo(id) * ra(ll,k)) **2
              end if
             enddo
          else
            rterms(id) = bad_value
          end if
          enddo
        end if
  
        if (platform_name(1:5) .eq. 'Aqua') then
          do id = 1,nsct 
          if(krto(id) .ne. 0 .and. id .ne. 3) then
            rterms(id) = 0.0
            ll = lev(id)
            do k = 1, nbct-1
              rterms(id) = rterms(id) + (robs(k) - rwarm(k) -
     &                          amo(id) * ra(ll,k)) **2
             enddo
          else
            rterms(id) = bad_value
          end if
          enddo
        end if
  
c ...   ---------------------------------------------------------------
  
c ...   Find the cloud height which gave the smallest error.
        ngch = 0
        rtemin = 999999.9
        do k = 1, ratio_sum
c ...  get ratio number of Terra or Aqua   
          id = ratio_sat(k) 
            if(krto(id) .ne. 0 ) then
c ...       Count the number of valid cloud heights.
            ngch = ngch + 1
            if(rterms(id) .lt. rtemin) then
              rtemin = rterms(id)
              lco2 = lev(id)
              ipco2 = id 
            end if
c ...       Save individual cloud height values for output.
            pco2(id) = pp(lev(id))
            cloudtop_pres_from_ratios(iout,jout,id) =
     &                          (nint(pco2(id) / 5.0)) * 5.0
           end if
        enddo

c ...   ---------------------------------------------------------------

c ...   Write debug information about final cloud height selection.
        if(debug .gt. 0) then
         if(line .eq. 6 .and. pixel .eq. 676) then
          write(h_output,'(1x)')
          write(h_output,'(''Cloud height selection'')')
          write(h_output,'(''MODIS channel combinations:'')')
          write(h_output,'(8x,8(i7,''/'',i2))') (mbnds(kch(ii,1)),
     &       mbnds(kch(ii,2)),ii=1,nsct)
          write(h_output,'(''heights '',8f12.3)') (pco2(ii),ii=1,nsct)
          write(h_output,'(''eff emiss '',8f10.3)') (amo(ii),ii=1,nsct)
          write(h_output,'(''errors    '',8f10.3)')
     &       (rterms(ii),ii=1,nsct)
         end if
        end if

c ...   ---------------------------------------------------------------
c ...   Check if there were any valid cloud heights.
        if(ngch .ne. 0) then

c ...     Save best cloud height for output.
          pfco2 = pp(lco2)
          ecaco2 = amo(ipco2)

c ...     Prepare final cloud height for output and set appropritate
c ...     flags.
c ...     Round cloud pressure to nearest 5 mb.
            pct = (nint(pfco2 / 5.0)) * 5.0
            eca = ecaco2
            ip = ipco2
            tct = tp(lco2)
            if(eca .le. 0.95) then
              ci_flag = 2
            else
              ci_flag = 1
            end if
            if(pct .lt. 400.0) then
              hc_flag = 2
            else
              hc_flag = 1
            end if

c ...       Fill output arrays.
            cloud_h_method(iout,jout) = ip * 1.0
            cloudtop_pres(iout,jout) = pct
            cloudtop_temp(iout,jout) = tct
            cloudtop_eff_emi(iout,jout) = eca

        end if
  
c ...   ---------------------------------------------------------------
  
c      end if

      end if

c ... Write output debug information
      if(debug .gt. 0) then
       if(line .eq. 6 .and. pixel .eq. 676) then
        write(h_output,'(1x)')
        write(h_output,'(''Output from get_co2cld'')')
        write(h_output,'(''Tropopause height '',f10.3)') 
     &    height_tropopause(iout,jout)
        write(h_output,'(''Window cloud height '',f10.3)') 
     &    cloudtop_pre_ir(iout,jout)
        write(h_output,'(''Final CO2 cloud height '',f10.3)') 
     &    cloudtop_pres(iout,jout)
        write(h_output,'(''Cloud effective emissivity '',f10.3)') 
     &    cloudtop_eff_emi(iout,jout)
        write(h_output,'(''Cloud temperature '',f10.3)') 
     &    cloudtop_temp(iout,jout)
        write(h_output,'(''CO2 cloud height solutions '',8f10.3)') 
     &    (cloudtop_pres_from_ratios(iout,jout,jj),jj=1,nsct)
        write(h_output,'(''CO2 spectral cloud forcings'',8f10.3)') 
     &    (spec_cloud_forcing(iout,jout,jj),jj=1,nsct)
        write(h_output,'(''Index of solution chosen'',f10.3)') 
     &    cloud_h_method(iout,jout)
        write(h_output,'(1x)')
        write(h_output,'(''Flag values'')')
        write(h_output,'(''qual_flag,conf_flag,hc_flag,ci_flag '',
     &    4i5)') qual_flag,conf_flag,hc_flag,ci_flag
       end if
      end if

c ... -----------------------------------------------------------------
  
      return
      end
