#!/bin/csh -f

# This script is an example of how to set up processing for the 
# the DB MODIS cloud top properties product.

#  This script:
#  1) Extracts ancillary data
#  2) Destripes band 26 data (currently commented out)
#  3) Extracts binary radiance/reflectance, geolocation and 
#     metadata information from MODIS DAAC or direct broadcast
#     HDF files.
#  4) Runs the direct broadcast MODIS cloud mask software.
#  5) Runs the direct broadcast MODIS cloud top properties software.

# Originally written by Kathy Strabala 
# kathy.strabala@ssec.wisc.edu April 2002.

# Updated January 2003 - Sat Name argument added, directory locations modified
# Search for !!! for places where directoy locations are set and will
#  need to be changed for on-site automated processing.

# Updated June 2005 - Runs collection 5 algorithm 
#  Latest alglorithm requires binary Reynolds blended Sea Surface
#  Temperature ancillary file (oisst.yyyymmdd)
#  Note:  New file size check for ancillary NISE files
#  Note: The profiles code requires the data month to be passed
#  as one of the arguments.  This script currently sets this
#  value at 4 to match the test data set month.  There is an
#  example below on how you would automate the selection of the
#  month to be taken from the current date.
#  

#-------------------------------------------------------------------------------
# SETUP
#-------------------------------------------------------------------------------

# Check arguments
if ($#argv != 5) then
  echo "Usage: modis_cloudtop.csh SAT FIL1KM FILQKM FILGEO OUTDIR"
  echo "where"
  echo "SAT is the MODIS platform name (Terra or Aqua)"
  echo "FIL1KM is MODIS L1B 1000 meter resolution HDF image file"
  echo "FILQKM is MODIS L1B 250  meter resolution HDF image file"
  echo "FILGEO is MODIS L1B 1000 meter resolution HDF geolocation file"
  echo "OUTDIR is the directory where products will be stored"
  exit(-1)
endif

# Extract file names
set SAT = $argv[1]
set FIL1KM = $argv[2]
set FILQKM = $argv[3]
set FILGEO = $argv[4]
set OUTDIR = $argv[5]

# Set root of output file name (e.g. 't1.02001.1815')
set ROOT = $FIL1KM:r
set ROOT = $ROOT:r
set ROOT = $ROOT:t

#Convert Satellite name to standard that threshold file will recognize
set fl=`echo $SAT|cut -c1`
if ( "$fl" == "T"  || "$fl" == "t") then
   set SAT = "Terra"
else if ( "$fl" == "A"  || "$fl" == "a" ) then
     set SAT = "Aqua"
else
     echo "Satellite name incorrect" $SAT
     exit 1
endif

#-------------------------------------------------------------------------------
# GET ANCILLARY DATA 
#  If you are not processing a data set where snow and ice
#  are present, you can delete or comment out the the 
#  NISE and NCEP_ICE portions of this section.

#  Works off of ancillary data sets in the given directory.

#  When processing, make sure you get the closest GDAS1 file to 
#  the overpass time.  There are 4 analyses available for each day.  

#  All dynamic ancillary data sets are available real-time at: 
#   ftp://terra.ssec.wisc.edu/pub/terra/ancillary
#-------------------------------------------------------------------------------

#!!!!!!! Set directory for ancillary data and source !!!!!!!!!!
set ANC_DIR = ../../data
set ANC_SRC = ../../ancillary/src

# Set default names for unpacked NISE files
set NISE_NORTH_BIN = ""
set NISE_SOUTH_BIN = ""

set NISE_FILE = `find $ANC_DIR -name "NISE_SSMIF13_????????.HDFEOS" -size +2180000c | sort | tail -1`
if ($NISE_FILE == "") then
  echo "No NISE files were found in "$ANC_DIR
  exit(1)
endif

# Point to and unpack the NISE data file
set NISE_NORTH_BIN = ${NISE_FILE:t}_NORTH.bin
set NISE_SOUTH_BIN = ${NISE_FILE:t}_SOUTH.bin
$ANC_SRC/extract_nsidc_nise.csh $NISE_FILE $NISE_NORTH_BIN $NISE_SOUTH_BIN
endif
  
# Set default name for unpacked NCEP ice file
set NCEP_ICE_BIN = ""

# Get name of correct ncep ICE file (should be only one in directory) 
set NCEP_ICE_FILE = `find $ANC_DIR -name "eng.??????" -size +226883c | sort | tail -1`
if ($NCEP_ICE_FILE == "") then
  echo "No NCEP ice files were found in "$ANC_DIR
  exit(1)
endif

set NCEP_ICE_BIN = ${NCEP_ICE_FILE:t}.bin
$ANC_SRC/extract_ncep_ice.csh $NCEP_ICE_FILE $NCEP_ICE_BIN

# Set default name for unpacked gdas1 file
set GDAS1_BIN = ""

# Find and unpack the GDAS1 file
set GDAS1_FILE = `find $ANC_DIR -name "gdas1.PGrbF00.??????.???" -size +22000000c | sort | tail -1`
if ($GDAS1_FILE == "") then
  echo "No GDAS1 model files were found in "$ANC_DIR
  exit(1)
endif

set GDAS1_BIN = ${GDAS1_FILE:t}.bin
$ANC_SRC/extract_ncep_gdas1.csh $GDAS1_FILE $GDAS1_BIN
endif

# Find the SST file - no extraction necessary for this binary file
set OISST_FILE =  `find $ANC_DIR -name "oisst.????????" | sort | tail -1`
set OISST = $OISST_FILE:t
/bin/cp ${ANC_DIR}/${OISST} ${OISST}

#------------------------------------------------------------------------------# RUN Band 26 DESTRIPING ALGORITHM
#------------------------------------------------------------------------------
#echo
#echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
#echo "Started MODIS 1KM destriping at "`date`
#echo "MODIS L1B 1KM: "$FIL1KM

#!!!!!! Set directory for destripe data and src !!!!!!!!!
#set SRCDIR = ../../destripe_band26/src
#set DATA_DIR = ../../data
#set COEFF_DIR = ../../destripe_band26/coeff
#$SRCDIR/correct_band26.exe $SAT $DATA_DIR/$FIL1KM $COEFF_DIR/latest.${SAT}.coefficients

#-------------------------------------------------------------------------------
# RUN CLOUD MASK ALGORITHM
#-------------------------------------------------------------------------------

# Print start message for processing log
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Started MODIS cloud mask processing at "`date`
echo "MODIS L1B 1KM: "$FIL1KM
echo "MODIS L1B QKM: "$FILQKM
echo "MODIS L1B GEO: "$FILGEO

# Extract flat files
echo "Started flatfile extraction at "`date`

#!!!!!! Set directory for flatfile extraction data and src !!!!!!!!!
set SRCDIR = ../../flatfile/src
set DATA_DIR = ../../data

$SRCDIR/modis_extract_1km.exe $DATA_DIR/$FIL1KM $SAT ${ROOT}.1000m.img ${ROOT}.1000m.hdr
$SRCDIR/modis_extract_qkm.exe $DATA_DIR/$FILQKM $SAT ${ROOT}.250m.img ${ROOT}.250m.hdr
$SRCDIR/modis_extract_geo.exe $DATA_DIR/$FILGEO $SAT ${ROOT}.geo.img ${ROOT}.geo.hdr
$SRCDIR/modis_extract_met.exe $DATA_DIR/$FIL1KM $SAT ${ROOT}.met.img ${ROOT}.met.hdr
echo "Finished flatfile extraction at "`date`
 

# Set output file names
set OUTFIL      = $ROOT.mod35.img
set OUTFILQA    = $ROOT.mod35qa.img
set OUTFILHDR   = $ROOT.mod35.hdr
set OUTFILQAHDR = $ROOT.mod35qa.hdr
set DEBUG       = $ROOT.mod35debug.txt

# Create a configuration file from the template
#!!!Set cloud mask src directory!!!
set SRCDIR = ../../cloudmask/cmd
set TEMPLATE = $SRCDIR/cloudmask.cfg
/bin/sed \
  -e "s/FIL1KM_IMG/${ROOT}.1000m.img/g" \
  -e "s/FIL1KM_HDR/${ROOT}.1000m.hdr/g" \
  -e "s/FILQKM_IMG/${ROOT}.250m.img/g" \
  -e "s/FILQKM_HDR/${ROOT}.250m.hdr/g" \
  -e "s/FILGEO_IMG/${ROOT}.geo.img/g" \
  -e "s/FILGEO_HDR/${ROOT}.geo.hdr/g" \
  -e "s/FILMET_IMG/${ROOT}.met.img/g" \
  -e "s/FILMET_HDR/${ROOT}.met.hdr/g" \
  -e "s/NISE_NORTH/${NISE_NORTH_BIN}/g" \
  -e "s/NISE_SOUTH/${NISE_SOUTH_BIN}/g" \
  -e "s/NCEP_ICE/${NCEP_ICE_BIN}/g" \
  -e "s/NCEP_GDAS/${GDAS1_BIN}/g" \
  -e "s/oisst_weekly/${OISST}/g" \
  -e "s/..\/static/..\/..\/cloudmask\/static/g" \
  -e "s/THRESHOLD_FILE/..\/..\/cloudmask\/cmd\/thresholds.dat.${SAT}/g" \
  -e "s/MOD35.img/${OUTFIL}/g" \
  -e "s/MOD35QA.img/${OUTFILQA}/g" \
  -e "s/MOD35.hdr/${OUTFILHDR}/g" \
  -e "s/MOD35QA.hdr/${OUTFILQAHDR}/g" \
  -e "s/debug.dat/${DEBUG}/g" \
  $TEMPLATE > cloudmask.cfg

# Run the cloud mask
$SRCDIR/cloudmask.exe cloudmask.cfg $SAT

# Rename the configuration file
/bin/mv cloudmask.cfg $ROOT.mod35.cfg

#-------------------------------------------------------------------------------
# RUN THE CLOUD TOP PROPERTIES AND CLOUD PHASE ALGORITHM
#-------------------------------------------------------------------------------

# Set cloud top properties output file names
set OUTCTPFIL   = $ROOT.mod06.img
set OUTCTPQA    = $ROOT.mod06qa.img
set OUTCTPHDR   = $ROOT.mod06.hdr
set OUTCTPQAHDR = $ROOT.mod06qa.hdr
set CTPDEBUG    = $ROOT.mod06debug.txt

# Create a configuration file from the template
#!!!Set cloud top properties src directory!!!
set SRCDIR = ../src
set TEMPLATE = $SRCDIR/cloudtop.cfg
/bin/sed \
  -e "s/FIL1KM_IMG/${ROOT}.1000m.img/g" \
  -e "s/FIL1KM_HDR/${ROOT}.1000m.hdr/g" \
  -e "s/FILGEO_IMG/${ROOT}.geo.img/g" \
  -e "s/FILGEO_HDR/${ROOT}.geo.hdr/g" \
  -e "s/FILMET_IMG/${ROOT}.met.img/g" \
  -e "s/FILMET_HDR/${ROOT}.met.hdr/g" \
  -e "s/MOD35_IMG/${ROOT}.mod35.img/g" \
  -e "s/MOD35_HDR/${ROOT}.mod35.hdr/g" \
  -e "s/NCEP_GDAS/${GDAS1_BIN}/g" \
  -e "s/oisst_weekly/${OISST}/g" \
  -e "s/MOD06.img/${OUTCTPFIL}/g" \
  -e "s/MOD06QA.img/${OUTCTPQA}/g" \
  -e "s/MOD06.hdr/${OUTCTPHDR}/g" \
  -e "s/MOD06QA.hdr/${OUTCTPQAHDR}/g" \
  -e "s/ctpdebug.dat/${CTPDEBUG}/g" \
  $TEMPLATE > cloudtop.cfg

# Need to get the month of the data being processed.  For this example
# the month is set to 4. For automated processing, uncomment the next
# line and delete the one following to use the current month.
#setenv MONTH `date +%m`
setenv MONTH 4

echo "Month of dataset " ${MONTH}

$SRCDIR/cloudtop.exe cloudtop.cfg $SAT $MONTH

# Rename the configuration file
/bin/mv cloudtop.cfg $ROOT.mod06.cfg

# Move the output files to the product directory
/bin/mv $OUTFIL $OUTFILQA $OUTFILHDR $OUTFILQAHDR $DEBUG $ROOT.mod35.cfg $OUTDIR

/bin/mv $OUTCTPFIL $OUTCTPQA $OUTCTPHDR $OUTCTPQAHDR $CTPDEBUG $ROOT.mod06.cfg $OUTDIR

#-------------------------------------------------------------------------------
# CLEANUP AND EXIT
#-------------------------------------------------------------------------------

# Print end message for processing log
echo
echo ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
echo "Finished MODIS cloud top properties processing at "`date`

# Exit gracefully
exit(0)
